from typing import Dict, Text
from unittest.mock import Mock

from _pytest.monkeypatch import MonkeyPatch
from pytest_sanic.utils import TestClient
from rasax.community.services.data_service import DataService


async def test_create_entity_synonym_mapped_values(
    test_client: TestClient, auth_header: Dict[Text, Text], monkeypatch: MonkeyPatch
):
    mapped_values = ["one", "two", "three"]
    payload = [{"value": i} for i in mapped_values]
    synonym_id = 10
    project = "default"

    add_entity_synonym_mapped_values = Mock(return_value=payload)
    monkeypatch.setattr(
        DataService,
        "add_entity_synonym_mapped_values",
        add_entity_synonym_mapped_values,
    )

    response = await test_client.post(
        f"/api/projects/{project}/synonyms/{synonym_id}",
        json={"mapped_values": payload},
        headers=auth_header,
    )

    assert response.status == 201
    response_body = await response.json()
    assert response_body == payload
    add_entity_synonym_mapped_values.assert_called_once_with(
        project, synonym_id, mapped_values
    )


async def test_test_create_entity_synonym_mapped_values_error(
    test_client: TestClient, auth_header: Dict[Text, Text], monkeypatch: MonkeyPatch
):

    add_entity_synonym_mapped_values = Mock(side_effect=ValueError)
    monkeypatch.setattr(
        DataService,
        "add_entity_synonym_mapped_values",
        add_entity_synonym_mapped_values,
    )

    response = await test_client.post(
        f"/api/projects/default/synonyms/1",
        json={"mapped_values": {"value": "test"}},
        headers=auth_header,
    )

    assert response.status == 400
