import uuid
import json
import random
from typing import Dict, Text

import pytest
from sanic.testing import SanicTestClient
from sqlalchemy.orm import Session

from tests.unit.conftest import _user_event, _bot_event, _action_event
from rasax.community.database.conversation import Conversation
from rasax.community.services.event_service import EventService

CONVERSATIONS_URL = "/api/conversations"


@pytest.fixture()
def conversation(event_service: EventService) -> Text:
    sender_id = uuid.uuid4().hex

    events = [
        _bot_event(sender_id, 1000),
        _user_event(sender_id, 2000),
        _bot_event(sender_id, 3000),
        _action_event(sender_id, 4000),
    ]

    for event in events:
        event_service.save_event(json.dumps(event), sender_id=sender_id)

    event_service.add_flagged_message(sender_id, 2000)
    event_service.add_flagged_message(sender_id, 4000)

    return sender_id


async def test_events_metadata_id(
    test_client: SanicTestClient,
    auth_header: Dict[Text, Text],
    event_service: EventService,
    session: Session,
    conversation: Text,
):
    response = await test_client.get(
        f"{CONVERSATIONS_URL}/{conversation}", headers=auth_header
    )
    response_body = await response.json()
    received_ids = [e["metadata"]["rasa_x_id"] for e in response_body["events"]]

    conversation_entity = session.query(Conversation).get(conversation)
    expected_ids = [e.id for e in conversation_entity.events]

    assert received_ids == expected_ids


async def test_events_metadata_is_flagged(
    test_client: SanicTestClient,
    auth_header: Dict[Text, Text],
    event_service: EventService,
    session: Session,
    conversation: Text,
):
    response = await test_client.get(
        f"{CONVERSATIONS_URL}/{conversation}", headers=auth_header
    )
    response_body = await response.json()
    received_flagged_ids = [
        e["metadata"]["rasa_x_id"]
        for e in response_body["events"]
        if e["metadata"]["rasa_x_flagged"]
    ]

    conversation_entity = session.query(Conversation).get(conversation)
    expected_flagged_ids = [e.id for e in conversation_entity.events if e.is_flagged]

    assert received_flagged_ids == expected_flagged_ids
