from typing import Optional, List, Text, Union, Dict

from _pytest.monkeypatch import MonkeyPatch
from rasax.community import config
from rasax.community.services.feature_service import FeatureService


def test_get_default_feature(feature_service: FeatureService, monkeypatch: MonkeyPatch):
    test_feature = {
        "name": "show the save button for the model config",
        "enabled": False,
    }
    _mock_feature_flags(monkeypatch, [test_feature])
    features = feature_service.features()

    assert len(features) == 1
    assert features[0] == test_feature


def _mock_feature_flags(
    monkeypatch: MonkeyPatch,
    server_features: Optional[List[Dict[Text, Union[Text, bool]]]] = None,
    local_features: Optional[List[Dict[Text, Union[Text, bool]]]] = None,
) -> None:
    monkeypatch.setattr(config, "DEFAULT_LOCAL_FEATURE_FLAGS", local_features or [])
    monkeypatch.setattr(config, "DEFAULT_SERVER_FEATURE_FLAGS", server_features or [])


def test_set_feature(
    feature_service: FeatureService, monkeypatch: MonkeyPatch, metrics_events: List
):
    test_feature = {
        "name": "show the save button for the model config",
        "enabled": True,
    }
    feature_service.set_feature(test_feature)
    feature_service.commit()

    # Assert feature update was tracked
    assert metrics_events

    test_feature_default = test_feature.copy()
    test_feature_default["enabled"] = False

    _mock_feature_flags(monkeypatch, [test_feature_default])

    features = feature_service.features()

    assert len(features) == 1
    assert features[0] == test_feature


def test_set_non_existing_feature(feature_service: FeatureService):
    test_feature = {"name": "print money", "enabled": False}

    feature_service.set_feature(test_feature)

    assert feature_service.features() == feature_service._default_feature_flags()


def test_default_features(monkeypatch: MonkeyPatch):
    local_features = [{"name": "local", "enabled": True}]

    server_features = [{"name": "server", "enabled": True}]

    _mock_feature_flags(monkeypatch, server_features, local_features)

    monkeypatch.setattr(config, "LOCAL_MODE", True)
    assert FeatureService._default_feature_flags() == local_features

    monkeypatch.setattr(config, "LOCAL_MODE", False)
    assert FeatureService._default_feature_flags() == server_features
